####################################################################
#### Analysis code for publication (Part 2)
#### "Expected demographic and genetic declines not found in most zoo and aquarium populations"

#### Judy Che-Castaldo Aug. 2020

#### includes:
    #### Bayesian modeling to estimate change in demographic and genetic metrics over time
####################################################################


require(rjags)
# devtools::install_github("caseyyoungflesh/MCMCvis")
library(MCMCvis)


#------------------------------------------------------------
### read in data
    ## Raw data on SSP Animal Programs are owned by the Association of Zoos and Aquariums (AZA) and are available upon written request
    ## Anonymous datasets used
#------------------------------------------------------------
plotdat <- read.csv("plotdat_anon.csv", sep=",", header=T) 
initNmat <- read.csv("initNmat_anon.csv", sep=",", header=T)


### Create response variables, transform if needed
plotdat$GD1 <- plotdat$Gene.Diversity.Retained / 100
plotdat$GD <- log(plotdat$GD1/(1-plotdat$GD1))                            # logit transformation
plotdat$Inbreeding <- log((plotdat$"Mean.Inbreeding..F." + 0.001)/(1-(plotdat$"Mean.Inbreeding..F." + 0.001)))  # logit transformation
plotdat$found <- log(plotdat$"X..Current.Founders")
plotdat$found <- as.numeric(as.character(plotdat$found))
plotdat$FGE <- log(plotdat$"X..Current.FGEs")
plotdat$FGE <- as.numeric(as.character(plotdat$FGE))



#------------------------------------------------------------
### specify model: test whether need TAG effect and/or program-specific variances (or shape parameter in gamma models)
##### for N, fit using Gamma distribution
##### for TPS, tested Gamma vs normal distributions
##### for all other responses, used Normal
#------------------------------------------------------------

### Random slopes, intercepts, and variances model for all TAGs with TAG level effects, Gamma distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int_grand ~ dnorm(0, 0.0001)      # Mean hyperparameter for random intercepts        
    sigma_int_tag ~ dunif(0, 1000)       # SD hyperparameter for random intercept at tag level        
    tau_int_tag <- 1/sigma_int_tag^2
    sigma_int_program ~ dunif(0, 1000)   # SD hyperparameter for random intercepts at program level         
    tau_int_program <- 1/sigma_int_program^2
    
    mu_slope_grand ~ dnorm(0, 0.001)        # Mean hyperparameter for random slopes
    sigma_slope_tag ~ dunif(0, 1000)        # SD hyperparameter for slopes at tag level
    tau_slope_tag <- 1/sigma_slope_tag^2
    sigma_slope_program ~ dunif(0, 1000)    # SD hyperparameter for slopes at program level
    tau_slope_program <- 1/sigma_slope_program^2
    
    alpha_shape ~ dunif(0, 10)    # hyperparameter 1 for random shape
    beta_shape ~ dunif(0, 10)    # hyperparameter 2 for random shape
    
    for (k in 1:n_tag) { 
      eta[k] ~ dnorm(mu_int_grand, tau_int_tag)        # Random intercepts at tag level
      gamma[k] ~ dnorm(mu_slope_grand, tau_slope_tag)  # Random slopes at tag level
    }
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(eta[tag[j]], tau_int_program)     # Random intercepts at program level
      beta[j] ~ dnorm(gamma[tag[j]], tau_slope_program)    # Random slopes at program level
      shape[j] ~ dgamma(alpha_shape, beta_shape)

      theta[j] <- exp(beta[j])            # derived quantities to get slope in odds ratio
      # zeta[j] <- (theta[j] - 1)*initN[j]      # include change in unit of number of indivdiuals
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dgamma(shape[species[i]], shape[species[i]]/exp(mu[i]))
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dgamma(shape[species[i]], shape[species[i]]/exp(mu[i]))
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])
    zNew <- sum(rsqNew[])   
    
  }
    ",fill=TRUE)
sink()


### Random slopes, intercepts, and variances model for all TAGs without TAG level effects, Gamma distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int ~ dnorm(0, 0.0001)    # Mean hyperparameter for random intercepts        
    sigma_int ~ dunif(0, 1000)   # SD hyperparameter for random intercepts          
    tau_int <- 1/sigma_int^2
    
    mu_slope ~ dnorm(0, 0.0001)     # Mean hyperparameter for random slopes
    sigma_slope ~ dunif(0, 100000)   # SD hyperparameter for slopes
    tau_slope <- 1/sigma_slope^2
    
    alpha_shape ~ dunif(0, 10)    # hyperparameter 1 for random shape
    beta_shape ~ dunif(0, 10)    # hyperparameter 2 for random shape
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(mu_int, tau_int)     # Random intercepts at program level
      beta[j] ~ dnorm(mu_slope, tau_slope)    # Random slopes at program level
      shape[j] ~ dgamma(alpha_shape, beta_shape)
    
      theta[j] <- exp(beta[j])            # derived quantities to get slope in odds ratio
      # zeta[j] <- (theta[j] - 1)*initN[j]      # include change in unit of number of indivdiuals
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dgamma(shape[species[i]], shape[species[i]]/exp(mu[i]))
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dgamma(shape[species[i]], shape[species[i]]/exp(mu[i]))
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])
    zNew <- sum(rsqNew[])   
    
  }
    ",fill=TRUE)
sink()



### Random slopes and intercepts model for all TAGs with TAG level effects but no program-specific variances, Gamma distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int_grand ~ dnorm(0, 0.0001)      # Mean hyperparameter for random intercepts        
    sigma_int_tag ~ dunif(0, 1000)       # SD hyperparameter for random intercept at tag level        
    tau_int_tag <- 1/sigma_int_tag^2
    sigma_int_program ~ dunif(0, 1000)   # SD hyperparameter for random intercepts at program level         
    tau_int_program <- 1/sigma_int_program^2
    
    mu_slope_grand ~ dnorm(0, 0.001)        # Mean hyperparameter for random slopes
    sigma_slope_tag ~ dunif(0, 1000)        # SD hyperparameter for slopes at tag level
    tau_slope_tag <- 1/sigma_slope_tag^2
    sigma_slope_program ~ dunif(0, 1000)    # SD hyperparameter for slopes at program level
    tau_slope_program <- 1/sigma_slope_program^2
    
    shape ~ dunif(0, 1000)   

    for (k in 1:n_tag) { 
      eta[k] ~ dnorm(mu_int_grand, tau_int_tag)        # Random intercepts at tag level
      gamma[k] ~ dnorm(mu_slope_grand, tau_slope_tag)  # Random slopes at tag level
    }
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(eta[tag[j]], tau_int_program)     # Random intercepts at program level
      beta[j] ~ dnorm(gamma[tag[j]], tau_slope_program)    # Random slopes at program level
    
      theta[j] <- exp(beta[j])            # derived quantities to get slope in odds ratio
      # zeta[j] <- (theta[j] - 1)*initN[j]      # include change in unit of number of indivdiuals
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dgamma(shape, shape/exp(mu[i]))
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dgamma(shape, shape/exp(mu[i]))
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])
    zNew <- sum(rsqNew[])   
    
    }
    ",fill=TRUE)
sink()




### Random slopes, intercepts, and variances model for all TAGs with TAG level effects, normally distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int_grand ~ dnorm(0, 0.0001)      # Mean hyperparameter for random intercepts        
    sigma_int_tag ~ dunif(0, 1000)       # SD hyperparameter for random intercept at tag level        
    tau_int_tag <- 1/sigma_int_tag^2
    sigma_int_program ~ dunif(0, 1000)   # SD hyperparameter for random intercepts at program level         
    tau_int_program <- 1/sigma_int_program^2
    
    mu_slope_grand ~ dnorm(0, 0.001)        # Mean hyperparameter for random slopes
    sigma_slope_tag ~ dunif(0, 1000)        # SD hyperparameter for slopes at tag level
    tau_slope_tag <- 1/sigma_slope_tag^2
    sigma_slope_program ~ dunif(0, 1000)    # SD hyperparameter for slopes at program level
    tau_slope_program <- 1/sigma_slope_program^2
    
    mu_sigma ~ dnorm(0, 0.001)     # Mean hyperparameter for random SDs
    sigma_sigma ~ dunif(0, 1000)   # SD hyperparameter for SDs
    tau_sigma <- 1/sigma_sigma^2
    
    for (k in 1:n_tag) { 
      eta[k] ~ dnorm(mu_int_grand, tau_int_tag)        # Random intercepts at tag level
      gamma[k] ~ dnorm(mu_slope_grand, tau_slope_tag)  # Random slopes at tag level
    }
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(eta[tag[j]], tau_int_program)     # Random intercepts at program level
      beta[j] ~ dnorm(gamma[tag[j]], tau_slope_program)    # Random slopes at program level
      tau_plan[j] ~ dnorm(mu_sigma, tau_sigma)            # Random SDs
      sigma_plan[j] <- sqrt(1/tau_plan[j])
    
      theta[j] <- exp(beta[j])            # derived quantities to get slope for logit or log transformed variables
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dnorm(mu[i], tau_plan[species[i]])    
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dnorm(mu[i], tau_plan[species[i]])
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
      zActual <- sum(rsqActual[])
      zNew <- sum(rsqNew[])   
    
  }
    ",fill=TRUE)
sink()



### Random slopes, intercepts, and variances model without TAG effects, normally distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int ~ dnorm(0, 0.0001)    # Mean hyperparameter for random intercepts        
    sigma_int ~ dunif(0, 1000)   # SD hyperparameter for random intercepts          
    tau_int <- 1/sigma_int^2
    
    mu_slope ~ dnorm(0, 0.0001)     # Mean hyperparameter for random slopes
    sigma_slope ~ dunif(0, 100000)   # SD hyperparameter for slopes
    tau_slope <- 1/sigma_slope^2
    
    mu_sigma ~ dnorm(0, 0.001)     # Mean hyperparameter for random SDs
    sigma_sigma ~ dunif(0, 1000)   # SD hyperparameter for SDs
    tau_sigma <- 1/sigma_sigma^2
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(mu_int, tau_int)     # Random intercepts
      beta[j] ~ dnorm(mu_slope, tau_slope)  # Random slopes
      tau_plan[j] ~ dnorm(mu_sigma, tau_sigma)   # Random SDs
      sigma_plan[j] <- sqrt(1/tau_plan[j])
    
      theta[j] <- exp(beta[j])              # derived quantities to get slope for logit or log transformed variables
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dnorm(mu[i], tau_plan[species[i]])
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dnorm(mu[i], tau_plan[species[i]])
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])   
    zNew <- sum(rsqNew[])   
    
    
  }
    ",fill=TRUE)
sink()


### Random slopes and intercepts model for all TAGs with TAG level effects but no program-specific variances, normally distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int_grand ~ dnorm(0, 0.0001)      # Mean hyperparameter for random intercepts        
    sigma_int_tag ~ dunif(0, 1000)       # SD hyperparameter for random intercept at tag level        
    tau_int_tag <- 1/sigma_int_tag^2
    sigma_int_program ~ dunif(0, 1000)   # SD hyperparameter for random intercepts at program level         
    tau_int_program <- 1/sigma_int_program^2
    
    mu_slope_grand ~ dnorm(0, 0.001)        # Mean hyperparameter for random slopes
    sigma_slope_tag ~ dunif(0, 1000)        # SD hyperparameter for slopes at tag level
    tau_slope_tag <- 1/sigma_slope_tag^2
    sigma_slope_program ~ dunif(0, 1000)    # SD hyperparameter for slopes at program level
    tau_slope_program <- 1/sigma_slope_program^2
    
    sigma_plan ~ dunif(0, 1000)   
    tau_plan <- 1/sigma_plan^2

    for (k in 1:n_tag) { 
      eta[k] ~ dnorm(mu_int_grand, tau_int_tag)        # Random intercepts at tag level
      gamma[k] ~ dnorm(mu_slope_grand, tau_slope_tag)  # Random slopes at tag level
    }
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(eta[tag[j]], tau_int_program)     # Random intercepts at program level
      beta[j] ~ dnorm(gamma[tag[j]], tau_slope_program)    # Random slopes at program level
    
      theta[j] <- exp(beta[j])            # derived quantities to get slope for logit or log transformed variables
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dnorm(mu[i], tau_plan)    
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dnorm(mu[i], tau_plan)
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])
    zNew <- sum(rsqNew[])   
    
    }
    ",fill=TRUE)
sink()


### Random slopes and intercepts model for all TAGs without TAG level effects and no program-specific variances, normally distributed
sink("model")
cat("
    model {
    
    # PRIORS
    mu_int ~ dnorm(0, 0.0001)    # Mean hyperparameter for random intercepts        
    sigma_int ~ dunif(0, 1000)   # SD hyperparameter for random intercepts          
    tau_int <- 1/sigma_int^2
    
    mu_slope ~ dnorm(0, 0.0001)     # Mean hyperparameter for random slopes
    sigma_slope ~ dunif(0, 100000)   # SD hyperparameter for slopes
    tau_slope <- 1/sigma_slope^2
    
    sigma_plan ~ dunif(0, 1000)   
    tau_plan <- 1/sigma_plan^2
    
    for (j in 1:n_spp) {
      alpha[j] ~ dnorm(mu_int, tau_int)     # Random intercepts
      beta[j] ~ dnorm(mu_slope, tau_slope)  # Random slopes

      theta[j] <- exp(beta[j])              # derived quantities to get slope for logit or log transformed variables
    }
    
    # LIKELIHOOD
    for (i in 1:n_obs) {
      mu[i] <- alpha[species[i]] + beta[species[i]]*time[i]
      y[i] ~ dnorm(mu[i], tau_plan)
    
      rsqActual[i] <- pow(y[i] - mu[i], 2)
      rnew[i] ~ dnorm(mu[i], tau_plan)
      rsqNew[i] <- pow(rnew[i] - mu[i], 2)
    }
    
    # posterior predictive check
    zActual <- sum(rsqActual[])   
    zNew <- sum(rsqNew[])   
    
    
    }
    ",fill=TRUE)
sink()




#------------------------------------------------------------
### set up model run
#------------------------------------------------------------

# specify data; first subset to omit NAs in response
  ## 7 responses:
  ## N (include initN), abs("TPS...N") - track theta and zeta
  ## log(lambda5yr) - track beta
  ## GD, Inbreeding, found, FGE - track beta and theta

plotdat1 <- droplevels(subset(plotdat, is.na(FGE)==FALSE & is.infinite(FGE)==FALSE))     
n_spp <- length(unique(plotdat1$Program1))
sppset <- as.character(unique(plotdat1$Program1))
plotdat1$yrsmgtC <- plotdat1$yrsmgt - mean(plotdat1$yrsmgt, na.rm=TRUE)      # center time variable
dd <- droplevels(subset(initNmat, Program1 %in% plotdat1$Program1))
dd$TAG1 <- as.numeric(dd$TAG)
n_tag <- length(unique(dd$TAG1))

## for gamma TPS model only:
plotdat1$"TPS...N"[(which(plotdat1$TPS...N == 0))] <- 0.1


### check that species and TAGs matched
sppset[218:225]
unique(plotdat1$Program1)[218:225]
dd$Program1[218:225]
dd$TAG[218:225]
as.numeric(dd$TAG[218:225])
dd$TAG1[218:225]


# set up analysis dataset
dataB = list(y=plotdat1$FGE, species=as.numeric(plotdat1$Program1), time=plotdat1$yrsmgtC, n_spp=n_spp,  n_obs=nrow(plotdat1), n_tag=n_tag, tag=dd$TAG1 )     	# xnew=xnew, initN=dd$initN, n_tag=n_tag, tag=dd$TAG1


## parameters to track:
# N including initN to standardize population size, abs("TPS...N") - track theta and zeta; 
# log(lambda5yr) - track beta
# GD, Inbreeding, found, FGE - track beta and theta; 


# with TAG effects
params <- c("alpha", "beta", "theta", "eta", "gamma", "mu_int_grand", "sigma_int_tag", "sigma_int_program", 
            "mu_slope_grand", "sigma_slope_tag", "sigma_slope_program", "sigma_plan", "zActual", "zNew")       # "theta", "zeta", "sigma_plan", "tau_plan", "mu_sigma", "sigma_sigma", 

# initial values
inits <- function() {
  list(alpha=rnorm(n_spp, 0, 1), beta=rnorm(n_spp, 0, 1), mu_int_grand=rnorm(1, 0, 1), sigma_int_tag=rlnorm(1), sigma_int_program=rlnorm(1), 
       mu_slope_grand=rnorm(1, 0, 1), sigma_slope_tag=rlnorm(1), sigma_slope_program=rlnorm(1), sigma_plan=runif(1,0,1) )  # , mu_sigma=runif(1,0,0.5), sigma_sigma=runif(1,0,0.5), sigma_plan=runif(1,0,1), alpha_shape=runif(1,0,5), beta_shape=runif(1,0,1)
}

# Better initial values
inits <- function() {
  list(alpha=rnorm(n_spp, 0, 1), beta=rnorm(n_spp, 0, 1), mu_int_grand=runif(1,1.7,2.1), sigma_int_tag=runif(1,0.3,0.7), sigma_int_program=runif(1,0.55,0.7), 
       mu_slope_grand=runif(1,0,0.015), sigma_slope_tag=runif(1,0,0.025), sigma_slope_program=runif(1,0.02,0.04), sigma_plan=runif(1,0.16,0.19) )  # mu_sigma=runif(1,120,200), sigma_sigma=runif(1,80,140), alpha_shape=runif(1,0,5), beta_shape=runif(1,0,1)
}



# no TAG effects
params <- c("alpha", "beta", "theta", "shape", "mu_int", "sigma_int", "mu_slope", "sigma_slope", "alpha_shape", "beta_shape", "zActual", "zNew")  # "theta", "zeta", "alpha_shape", "beta_shape", 

# initial values
inits <- function() {
  list(alpha=rnorm(n_spp, 0, 1), beta=rnorm(n_spp, 0, 1), mu_int=rnorm(1, 0, 1), sigma_int=rlnorm(1), 
       mu_slope=rnorm(1, 0, 1), sigma_slope=rlnorm(1), alpha_shape=runif(1,0,5), beta_shape=runif(1,0,1)  ) 
}

# Better initial values
inits <- function() {
  list(alpha=rnorm(n_spp, 0, 1), beta=rnorm(n_spp, 0, 1), mu_int=runif(1,3.3,3.5), sigma_int=runif(1,0.4,0.65),  
       mu_slope=runif(1,-0.04,-0.01), sigma_slope=runif(1,0.05,0.08), alpha_shape=runif(1,0.6,1.6), beta_shape=runif(1,0.05,0.3) )    
}

MCMCtrace(zc, params = c("mu_int", "sigma_int", "mu_slope", "sigma_slope", "alpha_shape", "beta_shape"))  



# no TAG effects, no random variances
params <- c("alpha", "beta", "theta", "mu_int", "sigma_int", "mu_slope", "sigma_slope", "sigma_plan", "zActual", "zNew")   # "theta", 

# initial values
inits <- function() {
  list(alpha=rnorm(n_spp, 0, 1), beta=rnorm(n_spp, 0, 1), mu_int=rnorm(1, 0, 1), sigma_int=rlnorm(1),  
       mu_slope=rnorm(1, 0, 1), sigma_slope=rlnorm(1), sigma_plan=rlnorm(1) )    
}

MCMCtrace(zc, params = c("mu_int", "sigma_int", "mu_slope", "sigma_slope", "sigma_plan"))  




nAdapt=5000		# number of iterations for choosing the sampler; 10000
nUpdate=5000		# number of iterations for burn-in; 10000
nIter=50000		  # number of iterations to store, to make the "rug"; 150000 and thin 10
nc=3

# set up MCMC chain
jm = jags.model("model", data=dataB, inits=inits, n.adapt=nAdapt, n.chains=nc)  

# burn-in the chain
update(jm, n.iter=nUpdate)

# generate coda object
zc = coda.samples(jm, variable.names=params, n.iter=nIter, thin=10) 		



MCMCtrace(zc, params = c("mu_int_grand", "sigma_int_tag", "sigma_int_program", "mu_slope_grand", "sigma_slope_tag", "sigma_slope_program", "sigma_plan"))  # "alpha_shape", "beta_shape", "mu_sigma", "sigma_sigma", "sigma_plan"
MCMCsummary(zc, params = c("mu_int_grand", "sigma_int_tag", "sigma_int_program", "mu_slope_grand", "sigma_slope_tag", "sigma_slope_program", "mu_sigma", "sigma_sigma") , n.eff=TRUE)


gelman.diag(zc)




## save summary files for plotting
zcsumN <- summary(zcN)
zcsumTPS <- summary(zcTPSnorm1var)
zcsumLam <- summary(zcLam)
zcsumGD <- summary(zcGDTag1var)
zcsumInb <- summary(zcInb1var)
zcsumFound <- summary(zcFoundTag1var)
zcsumFGE <- summary(zcFGETag1var)

save(zcsumN, file="zcsumN.rda")
save(zcsumTPS, file="zcsumTPS.rda")
save(zcsumLam, file="zcsumLam.rda")
save(zcsumGD, file="zcsumGD.rda")
save(zcsumInb, file="zcsumInb.rda")
save(zcsumFound, file="zcsumFound.rda")
save(zcsumFGE, file="zcsumFGE.rda")

