####################################################################
#### Analysis code for publication (Part 1)
#### "Expected demographic and genetic declines not found in most zoo and aquarium populations"

#### Judy Che-Castaldo Aug. 2020

#### includes:
    #### Recent plans summary statistics
        ## WebTables 1 and 2

    #### Trends over time analysis: ctree and cforest analysis of slope estimates from Bayesian analysis
        ## Conditional inference tree plots in figure 4 and webfigure 2
        ## Partial dependence plot in webfigure 3

    #### Creating graphs for manuscript 
        ## Change over time summary plots (Figure 2)
        ## Compare expected declines in GD based on drift against observed (Figure 3)
        ## Histograms of observed time spans (Webfigure 1)
        ## Mean change in GD, Founders, FGE by taxon (Webfigure 4)
####################################################################

library(tidyverse)
library(cowplot)
library(party)
library(randomForest)
library(MASS)


#------------------------------------------------------------
#### Recent plans summary statistics
#------------------------------------------------------------
tabdat <- read.csv("recentprogramsdat_anon.csv", sep=",", header=T)  


### Summary table 1: summary statistics
tab <- aggregate(tabdat$N, list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))
tab1 <- cbind(tab, aggregate(tabdat$N, list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$N, list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$planage, list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"TPS...N", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"TPS...N", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"TPS...N", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"lambda5yr", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"lambda5yr", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x, 
              aggregate(tabdat$"lambda5yr", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"Gene.Diversity.Retained", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"Gene.Diversity.Retained", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"Gene.Diversity.Retained", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"Mean.Inbreeding..F.", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"Mean.Inbreeding..F.", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Current.Founders", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Current.Founders", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Current.FGEs", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Current.FGEs", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Known.After.Exclusions.Assumptions", list(tabdat$Program.Type, tabdat$Taxon), FUN=mean, na.rm=TRUE)$x,
              aggregate(tabdat$"X..Known.After.Exclusions.Assumptions", list(tabdat$Program.Type, tabdat$Taxon), FUN=sd, na.rm=TRUE)$x)
colnames(tab1) <- c("Designation", "Class", "Programs", "Pop Size", "Pop Size SD", "Plan Age", "Target N", "From Target", "From Target SD", 
                    "5yr Lambda N", "5yr Lambda", "5yr Lambda SD", "Genetics N", "GD current", "GD SD", "Inbreeding", "Inbreeding SD", 
                    "Current Founders", "Founders SD", "Current FGE", "FGE SD", "Knownness", "Knownness SD")

tab1


### Summary table 2: percentages above thresholds
tab2 <- aggregate(tabdat$N, list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))
tab3 <- cbind(tab2, aggregate(tabdat$N, list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(x>200))/length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"lambda5yr", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"lambda5yr", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(x>=1))/length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"Gene.Diversity.Retained", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"Gene.Diversity.Retained", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(x>=90))/length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"X..Current.Founders", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"X..Current.Founders", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(x>=20))/length(which(is.na(x)==FALSE)))$x,
              aggregate(tabdat$"X..Current.Founders", list(tabdat$Program.Type, tabdat$Taxon), FUN=function(x) length(which(x>=50))/length(which(is.na(x)==FALSE)))$x)
colnames(tab3) <- c("Designation", "Class", "Programs", "N>200", "lambda N", "lambda >=1", "GD N", "GD >=90%", "founders N", "founders >=20", "founders >=50") 

tab3


#------------------------------------------------------------
#### Trends over time analysis: ctree and cforest analysis of slope estimates from Bayesian analysis
#------------------------------------------------------------
Bayes2 <- read.csv("BayesAnalysis_anon.csv", sep=",", header=T)  
  Bayes2 <- Bayes2 %>% mutate(across(where(is.character), as_factor))
  names(Bayes2)

### set up dataset, eliminate NAs
# responses (continuous): demography 2 - N; 4 - TPS; 5 - lambda; 
  ### genetics 6 - GD; 7 - Inbreeding; 8 - Founders; 8 - FGE
# predictors: programmatic 26,28,30,31,33,34; biology 27,29,32,35:38,40,43


dat5 <- Bayes2[,c(4, 26:38,40,43)]  
colnames(dat5)[1] <- "y"
dat5 <- subset(dat5, is.na(dat5[,1])==FALSE)

nas <- as.vector(rowSums(is.na(dat5)))  
length(which(nas<1))
datuse <- dat5[nas<1,]
datuse <- datuse[which(is.infinite(datuse$y)==FALSE),]
datuse <- droplevels(datuse)


#------------------------------------------------------------
# Conditional Inference Tree

#### creates inference tree plots in figure 4 and webfigure 2
#------------------------------------------------------------
trait.ct <- ctree(y ~ ., data = datuse, controls = ctree_control(mincriterion = 0.95))
nodes(trait.ct, 1)

# plot results
plot(trait.ct)   


# percent variance explained by model
y_hat <- predict(trait.ct, newdata=datuse)
err <- mean((datuse$y - as.vector(y_hat))^2)
err
(1-err/var(datuse$y))*100

var(datuse$y)


#------------------------------------------------------------
# Conditional Random Forest

#### creates partial dependence plot in webfigure 3
#------------------------------------------------------------
set.seed(125)
trait.crf <- cforest(y ~ ., data = datuse, control = cforest_unbiased(ntree=1000, mtry=4))

# variable importance
set.seed(411)
myvarimp <- varimp(trait.crf, conditional=TRUE)    # conditional is esp. important for correlated variables


# plot results
ord <- order(myvarimp, decreasing=TRUE)
par(mar=c(7,5,3,1))
barplot(myvarimp[ord], space=0.5, xaxt="n", ylab = "Mean decrease in accuracy", cex.axis=0.7)
text((1:15)*1.5-0.5, par("usr")[3]-0.0000001, labels=names(myvarimp)[ord], srt=40, adj=1, xpd=TRUE, cex=0.7)
abline(h=0)

# partial dependence plot
partialPlot.party(trait.crf, data, Mean.Generation.Length.x, rug=TRUE)    ## first run code for partialPlot.party function at end of code


# percent variance explained by model
y_hat1 <- predict(trait.crf, newdata=NULL)
err1 <- mean((datuse$y - as.vector(y_hat1))^2)
err1
(1-err1/var(datuse$y))*100

var(datuse$y)




#------------------------------------------------------------
#### Creating graphs for manuscript
#------------------------------------------------------------

#------------ Change over time summary plots (Figure 2) ------------#
Bayes2 <- subset(Bayes2, propGT >= 2)       # propGT >= 2 for those with at least 2 generation lengths observed

Metric1 <- c(rep("PopulationSize" , 3) , rep("SizeFromTarget" , 3) , rep("GrowthRate" , 3), rep("GeneDiversity" , 3), rep("Inbreeding" , 3), rep("Founders" , 3), rep("FGEs" , 3) )    # rep("MeanKinship" , 3), 
ChangeType <- rep(c("Decreasing", "Increasing", "NoChange") , 7)
values1 <- c(prop.table(table(Bayes2$PopSizeY))[ order(names(prop.table(table(Bayes2$PopSizeY))))],
             prop.table(table(Bayes2$FromTargetY))[ order(names(prop.table(table(Bayes2$FromTargetY))))],
             prop.table(table(Bayes2$LambdaY))[ order(names(prop.table(table(Bayes2$LambdaY))))],
             prop.table(table(Bayes2$GDY))[ order(names(prop.table(table(Bayes2$GDY))))],
             prop.table(table(Bayes2$InbY))[ order(names(prop.table(table(Bayes2$InbY))))],
             prop.table(table(Bayes2$FoundY))[ order(names(prop.table(table(Bayes2$FoundY))))],
             prop.table(table(Bayes2$FGEY))[ order(names(prop.table(table(Bayes2$FGEY))))] )
 
Btab3 <- data.frame(Metric1, ChangeType, values1)
Btab3$Metric1 <- factor(Btab3$Metric1, levels = Btab3$Metric1[c(1,4,7,10,13,16,19)])   
my.labels <- paste0(c("PopulationSize", "SizeFromTarget", "GrowthRate", "GeneDiversity", "Inbreeding", "Founders", "FGEs"), 
                    "\n (", c("371", "371", "368", "357", "358", "359", "358"), ")")    # c("371", "371", "368", "357", "358", "359", "358");  c("42", "42", "42", "42", "42", "42", "42", "42") for >2 propGT data only


plot3 = ggplot(Btab3, aes(fill=ChangeType, x=Metric1, y=values1)) + geom_bar(position="dodge", stat = "identity") +
  scale_fill_manual(values=c("gray", "#007765", "#6AA341")) +
  scale_x_discrete(labels = my.labels) +
  xlab(NULL) +
  #xlab("Metric") +
  ylab("Proportion of Populations") +
  theme_bw() + 
  #theme(legend.position = "none") + 
  theme(legend.position = "bottom", legend.direction = "horizontal", legend.text = element_text(size=14), legend.title = element_text(size=14)) + 
  theme(axis.text.x  = element_text(size=14, color="black", face = "bold"),
        axis.text.y  = element_text(size=14, color="black", face = "bold"),
        axis.title.x = element_text(size=16, vjust=-2, color="black", face = "bold"),
        axis.title.y = element_text(size=16, vjust=5, color="black", face = "bold"),
        panel.grid.minor.y=element_blank(),
        panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank(),
        plot.margin = unit(c(0.5,1,0.5,1),"cm"))


## plot3 = uisng full Bayes2, plot4 = using subset with propGT>=2
plot_grid(plot3, plot4, labels = "auto", nrow=2, label_size = 20) 



#------------ Compare expected declines in GD based on drift against observed (Figure 3) ------------#
GDmatsum = Bayes2 %>%   
  # filter(propGT > 2)  %>%   ## create GDmatsum2 (>2 generations observed)
  filter(is.na(ObsExp)==FALSE)  %>%   
  group_by(ObsExp) %>%
  summarise(
    n = n()
  ) %>%
  mutate(prop = n/sum(n)) 


plotA <- ggplot(GDmatsum) + aes(ObsExp, prop) + 
  geom_bar(position = "dodge", stat = "identity", fill="royalblue4") + 
  labs(x="Observed vs. Expected Decline (All, N=297)", y="Proportion of Programs") +
  scale_x_discrete(labels=c("As Expected", "Less Decline", "More Decline")) +
  ylim(0,0.8) +
  theme_bw() + 
  theme(axis.text.x  = element_text(size=14, color="black", face = "bold"),
        axis.text.y  = element_text(size=14, color="black", face = "bold"),
        axis.title.x = element_text(size=16, vjust=-2, color="black", face = "bold"),
        axis.title.y = element_text(size=16, vjust=4, color="black", face = "bold"),
        panel.grid.minor.y=element_blank(),
        panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank(),
        plot.margin = unit(c(0.5,0.5,0.5,0.5),"cm"))

plotB <- ggplot(GDmatsum2) + aes(ObsExp, prop) + 
  geom_bar(position = "dodge", stat = "identity", fill="skyblue") + 
  labs(x="Observed vs. Expected Decline (>2 GT; N=29)", y="") +
  scale_x_discrete(labels=c("As Expected", "Less Decline", "More Decline")) +
  ylim(0,0.8) +
  theme_bw() + 
  theme(axis.text.x  = element_text(size=14, color="black", face = "bold"),
        axis.text.y  = element_text(size=14, color="black", face = "bold"),
        axis.title.x = element_text(size=16, vjust=-2, color="black", face = "bold"),
        axis.title.y = element_text(size=16, vjust=5, color="black", face = "bold"),
        panel.grid.minor.y=element_blank(),
        panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank(),
        plot.margin = unit(c(0.5,0.5,0.5,0.5),"cm"))

plot_grid(plotA, plotB, labels = "auto", label_size = 20)   




#------------ Histograms of observed time spans (Webfigure 1)------------#
plot1 <- ggplot(Bayes2, aes(x=ObsYrs)) +   
  geom_histogram(color="gray75", fill="#007765", binwidth = 0.8) +  
  xlab("Years Observed") +
  ylab("Count") +
  theme_bw(base_size = 14) + 
  theme(panel.grid.minor.y=element_blank(),
        panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank())

plot2 <- ggplot(Bayes2, aes(x=propGT)) +  
  geom_histogram(color="gray75", fill="#6AA341", binwidth = 0.4) +  
  xlab("Generation Times Observed") +
  ylab("Count") +
  theme_bw(base_size = 14) + 
  theme(panel.grid.minor.y=element_blank(),
        panel.grid.minor.x=element_blank(),
        panel.grid.major.x=element_blank())

plot_grid(plot1, plot2, labels = "auto")




#------------ Mean change in GD, Founders, FGE by taxon (Webfigure 4) ------------#
Btab2 <- Bayes2[,c(1,27,6,10,12)] %>% 
  pivot_longer(c(GD, Founders, FGEs), names_to = "Metric", values_to = "values")

ggplot(Btab2, aes(x=Metric, y=values, fill=Taxon.x)) + 
  geom_boxplot(outlier.shape=NA) + 
  xlab("Metric") +
  ylab("Change in Genetic Metrics (slopes)") +
  ylim(0.9,1.15) +
  labs(fill = "Taxon") + 
  scale_fill_manual(values=c("#007765", "gray", "#6AA341")) +
  theme_bw(base_size = 16)





#------------------------------------------------------------
#### partialPlot.party function code
## from R package party version 0.9-98
#------------------------------------------------------------

partialPlot.party <-
  function (x, newdata, x.var, which.class, weights, plot = TRUE, add = FALSE,
            n.pt = min(length(unique(newdata[, xname])), 51), rug = TRUE,
            xlab = deparse(substitute(x.var)), ylab = "",
            main = paste("Partial Dependence on", deparse(substitute(x.var))),
            ...) 
  {
    classRF <- all(x@responses@is_nominal || x@responses@is_ordinal)
    
    x.var <- substitute(x.var)
    xname <- if (is.character(x.var)) x.var else {
      if (is.name(x.var)) deparse(x.var) else {
        eval(x.var)
      }
    }
    
    if (!xname %in% names(newdata))
      stop("variable", " ", xname, " ", "not known")
    xv <- newdata[, xname]
    n <- nrow(newdata)
    
    if (missing(weights)) weights <- rep(1, n)
    
    if (classRF) {
      if (missing(which.class)) {
        focus <- 1
      }
      else {
        focus <- charmatch(which.class, levels(x@responses@variables[[1]]))
        if (is.na(focus)) 
          stop(which.class, " ", "is not one of the class labels.")
      }
    }
    if (is.factor(xv)) { ### includes ordered
      x.pt <- levels(xv)
      y.pt <- numeric(length(x.pt))
      for (i in seq(along = x.pt)) {
        x.data <- newdata
        x.data[, xname] <- factor(rep(x.pt[i], n), levels = x.pt, ordered = is.ordered(xv))
        if (classRF) {
          pr <- treeresponse(x, newdata = x.data)
          pr <- matrix(unlist(pr), nrow = length(pr), byrow = TRUE)
          y.pt[i] <- weighted.mean(log(ifelse(pr[, focus] > 0,
                                              pr[, focus], 1)) -
                                     rowMeans(log(ifelse(pr > 0, pr, 1))),
                                   weights, na.rm=TRUE)
        } else y.pt[i] <- weighted.mean(predict(x, newdata = x.data), weights, na.rm = TRUE)
        
      }
      if (add) {
        points(1:length(x.pt), y.pt, type="h", lwd=2, ...)
      } else {
        if (plot) barplot(y.pt, width=rep(1, length(y.pt)), col="blue",
                          xlab = xlab, ylab = ylab, main=main,
                          names.arg=x.pt, ...)
      }
    } else {
      x.pt <- seq(min(xv), max(xv), length = n.pt)
      y.pt <- numeric(length(x.pt))
      for (i in seq(along = x.pt)) {
        x.data <- newdata
        x.data[, xname] <- rep(x.pt[i], n)
        class(x.data[, xname]) <- class(newdata[, xname])
        storage.mode(x.data[, xname]) <- storage.mode(newdata[, xname])
        if (classRF) {
          pr <- treeresponse(x, newdata = x.data)
          pr <- matrix(unlist(pr), nrow = length(pr), byrow = TRUE)
          y.pt[i] <- weighted.mean(log(ifelse(pr[, focus] == 0, 1, pr[, focus]))
                                   - rowMeans(log(ifelse(pr == 0, 1, pr))),
                                   weights, na.rm=TRUE)
        } else {
          y.pt[i] <- weighted.mean(predict(x, newdata = x.data), weights, na.rm=TRUE)
        }
      }
      if (add) {
        lines(x.pt, y.pt, ...)
      } else {
        if (plot) plot(x.pt, y.pt, type = "l", xlab=xlab, ylab=ylab,
                       main = main, ...)
      }
      if (rug && plot) {
        if (n.pt > 10) {
          rug(quantile(xv, seq(0.1, 0.9, by = 0.1)), side = 1)
        } else {
          rug(unique(xv, side = 1))
        }
      }
    }
    invisible(list(x = x.pt, y = y.pt))
    y.pt
  }

